#!/bin/bash

function usage {
    echo "Usage: $0 [--help] [--list <filename>] suffix_1 suffix_2"
    echo ""
    echo "--help        show this information"
    echo "--list        run only the benchmarks listed in file instead of all"
}

if [ $# -eq 0 ]; then
    usage;
    exit 1
fi

# parse arguments
expect_list_filename=0
suffix_1=""
suffix_2=""
for i in "$@"; do
    if [ $expect_list_filename -eq 1 ]; then
        list_filename="$i";
        expect_list_filename=0
    fi
    if [ $i = "--list" ]; then
        if [ "z$list_filename" = "z" ]; then
            expect_list_filename=1;
        else
            echo "'--list' repeated!";
            usage;
            exit 2;
        fi
    elif [ $i = "--help" ]; then
        if [ $# -ne 1 ]; then
            echo "'--help' given, ignoring other arguments";
        fi
        usage;
        exit 0;
    elif [ "z$suffix_1" == "z" ]; then
      suffix_1=$i
    elif [ "z$suffix_2" == "z" ]; then
      suffix_2=$i
    else
      echo "too many arguments!";
      exit 5;
    fi
done
if [ $expect_list_filename -eq 1 ]; then
    echo "no file name given for '--list'";
    usage;
    exit 3;
fi

if [ "z$list_filename" = "z" ]; then
    list_filename="utilities/benchmark_list";
fi
if [ ! -f "$list_filename" ]; then
    echo "file $list_filename does not exist";
    exit 4;
fi
benchmarks=`cat $list_filename`

## table output
echo "bench status.$suffix_1 time.$suffix_1 status.$suffix_2 time.$suffix_2"

for bench in $benchmarks; do
    dname=`dirname $bench`
    fname=`basename $bench`
    fnc=${fname/.c/}
    oldpwd=`pwd`
    cd "$dname"
    cat > Makefile <<EOF
SRC:=$fnc
include ${oldpwd}/common.mk
EOF
    diff "${fnc}.${suffix_1}.c" "${fnc}.${suffix_2}.c" >/dev/null
    dr=$?
    if [ $dr -eq 0 ]; then
	cd "$oldpwd";
        continue;
    else
    	echo -n "${fnc} "
        make "${fnc}.${suffix_1}.exe" >/dev/null 2>/dev/null
        build_ok_1=$?
        make "${fnc}.${suffix_2}.exe" >/dev/null 2>/dev/null
        build_ok_2=$?
        if [ $build_ok_1 -eq 0 ]; then
            "./${fnc}.${suffix_1}.exe" > /tmp/polybench_run 2>"${fnc}.${suffix_1}.out"
            t=`cat /tmp/polybench_run`
            dr=$[1 - $dr]
            echo -n "$dr $t"
        else
            echo -n "2 NA"
        fi
        if [ $build_ok_2 -eq 0 ]; then
            "./${fnc}.${suffix_2}.exe" > /tmp/polybench_run 2>"${fnc}.${suffix_2}.out"
            t=`cat /tmp/polybench_run`
            dr=$[1 - $dr]
            echo -n "$dr $t"
        else
            echo -n "2 NA"
        fi
    fi
    echo
    cd "$oldpwd"
done
